package SkeletonCart::Item;

use strict;
use warnings;
use utf8;
use MT;
use MT::Object;
use base qw(MT::Object);

use SkeletonCart;
use SkeletonCart::App;

__PACKAGE__->install_properties({
    column_defs => {
        id => 'integer not null auto_increment',
        blog_id => 'integer not null',
        entry_id => 'integer not null',
        item_id => { type => 'string', size => 255, },
        name => { type => 'string', size => 255, },
        price => { type => 'string', size => 255, },
        stock => { type => 'integer', },
        stock_message => { type => 'string', size => 255, },
        sales_status => { type => 'integer', },
        status_message => { type => 'string', size => 255, },
        stock_quantity => { type => 'integer', },
        group_id => { type => 'integer', },
        option_field_1 => { type => 'text', },
        option_field_2 => { type => 'text', },
        option_field_3 => { type => 'text', },
        option_field_4 => { type => 'text', },
        option_field_5 => { type => 'text', },
        weight => { type => 'string', size => 255, },
        active => { type => 'integer', },
    },
    indexes => {
        'entry_id' => 1,
        'item_id' => 1,
    },
    defaults => {
        group_id => 1,
        active => 0,
    },
    child_of => [ 'MT::Entry' ],
    audit => 1,
    datasource => 'sc_item',
    primary_key => 'id',
});

sub class_type {
    return 'sc_item';
}

sub class_label {
    return translate('Item');
}

sub class_label_plural {
    return translate('Items');
}

sub _normalize_number {
    my ($number) = @_;
    require Encode;
    require MT::I18N;
    my $enc = MT::I18N::guess_encoding($number);
    $number = Encode::decode($enc, $number)
        unless Encode::is_utf8($number);
    $number =~ tr/\x{ff10}-\x{ff19}\x{ff0c}\x{ff0e}\x{2212}/0-9,.\-/;
    $number =~ s/,//g;
    return $number;
}

sub pre_save_entry {
    my ($cb, $app, $entry, $org_entry) = @_;
    return 1 unless enabled() && (stock_enabled() || use_item_weight());
    my $id = $entry->id or return 1;
    my $perms = $app->permissions;
    my $user = $app->user;
        return 1 unless $perms->can_edit_entry($entry, $user, 1);
        
    # check quantity
    if(stock_enabled()) {
        my ($quantity, $qty_adjust);
        if ($app->mode eq 'save_entries') {
            $quantity = _normalize_number($app->param('item_stock_quantity_'.$id)) || 0;
            $qty_adjust = _normalize_number($app->param('item_stock_quantity_adjust_'.$id)) || 0;
        } else {
            $quantity = _normalize_number($app->param('item_stock_quantity')) || 0;
            $qty_adjust = _normalize_number($app->param('item_stock_quantity_adjust')) || 0;
        }
        return $cb->error(
            translate('Stock Quantity must be a number.')
        ) unless defined $quantity && length $quantity > 0 && $quantity =~ m!^[0-9]+$!;
        return $cb->error(
            translate('Stock Quantity Adjust must be a number.')
        ) unless defined $qty_adjust && length $qty_adjust > 0 && $qty_adjust =~ m!^[0-9]+$!;
    }

    if($app->mode ne 'save_entries') {
        # check price
        my $price = _normalize_number($app->param('item_price'));
        return $cb->error(
            translate('Price must be a number.')
        ) if defined $price && length $price > 0 && !($price =~ m!^\-?[0-9]+$!);

        # check weight
        if(use_item_weight()) {
            my $weight = _normalize_number($app->param('item_weight'));
            if(defined $weight && length $weight > 0) {
                require Scalar::Util;
                if (!Scalar::Util::looks_like_number($weight) || $weight < 0) {
                    return $cb->error(
                        translate('Item weight must be a positive number.')
                    );
                }
            }
        }

        # check required
        require MT::Entry;
        my $release = $entry->status == MT::Entry::RELEASE() ? 1 : 0;
        if($release) {
            my @params = qw/item_name item_price/;
            unshift @params, 'item_id' unless use_entry_id();
            foreach my $key (@params) {
                my $value = $app->param($key);
                unless(defined $value && length $value > 0) {
                    return $cb->error(
                        translate('[_1] is required.', translate($key))
                    );
                }
            }
        }
    }

    return 1;
}

sub post_save_entry {
    my ($cb, $app, $entry, $org_entry) = @_;

    return 1 unless enabled();
    my $entry_id = $entry->id or return 1;
    my $blog_id = $entry->blog_id or return 1;
    my $perms = $app->permissions;
    my $user = $app->user;
    return 1 unless $perms->can_edit_entry($entry, $user, 1);

    my $class = __PACKAGE__;

    # lock()
    $class->driver->sql("LOCK TABLES $class->table_name WRITE");

    eval {
        my $item;
        if($entry_id) {
            ($item) = SkeletonCart::Item->load({ entry_id => $entry_id });
        }
        unless($item) { 
            $item = SkeletonCart::Item->new();
        }
      
        my $names = $item->column_names;
        my %values = map { $_ => scalar $app->param('item_' . $_) } @$names;
        delete $values{id};
        delete $values{stock_quantity};
        $values{stock_quantity} = $item->stock_quantity;
        $values{entry_id} = $entry_id;
        $values{blog_id} = $blog_id;
        $values{item_id} = $app->param('item_id');

        # Adjust stock
        my $adj = $app->param('item_stock_quantity_adjust') || 0;
        my $adj_type = $app->param('item_stock_quantity_adjust_type') || 'add';
        if($adj != 0) {
            if($adj_type eq 'add') {
                $values{stock_quantity} += $adj;
            }
            elsif($adj_type eq 'reduce') {
                $values{stock_quantity} -= $adj;
            }
            if($values{stock_quantity} < 0) {
                $values{stock_quantity} = 0;
            }
        }

        require MT::Entry;
        $values{active} = $entry->status == MT::Entry::RELEASE() ? 1 : 0;
        $item->set_values(\%values);
    
        unless($item->save) {
            my $message = translate('Saving [_1] failed: [_2]', class_label(), $item->errstr);
            die $message;
        }
    };

    # unlock() 
    $class->driver->sql("UNLOCK TABLES");

    if($@) {
        return $cb->error($@);
    }

    return 1;
}

sub item_for_entry {
    my $entry = shift;
    my $class = __PACKAGE__;
    if($entry->can('cache_property')) {
        return $entry->cache_property(
            'sc_item',
            sub {
                my ($item) = $class->load({ entry_id => $entry->id });
                return $item;
            }
        );
    }
    else {
        my ($item) = $class->load({ entry_id => $entry->id });
        return $item;
    }
}

sub post_bulk_save_entries {
    my ($cb, $app, $entries) = @_;

    return 1 unless enabled();

    my $class = __PACKAGE__;
    # lock()
    $class->driver->sql("LOCK TABLES $class->table_name WRITE");

    eval {
        foreach my $pair (@$entries) {
            my $entry = $pair->{current};
            my $orig_entry = $pair->{original};

            my $entry_id = $entry->id
                or next;
            my $blog_id = $entry->blog_id
                or next;
            my $perms = $app->permissions;
            my $user = $app->user;
            return $app->permission_denied
                unless $perms->can_edit_entry($entry, $user);
        
            my ($item) = $class->load({ entry_id => $entry_id })
                or next;
            my $stock_quantity = $item->stock_quantity;
            # Adjust stock
            my $adj = $app->param('item_stock_quantity_adjust_' . $entry_id) || 0;
            my $adj_type = $app->param('item_stock_quantity_adjust_type_' . $entry_id) || 'add';
            if($adj != 0) {
                if($adj_type eq 'add') {
                    $stock_quantity += $adj;
                }
                elsif($adj_type eq 'reduce') {
                    $stock_quantity -= $adj;
                }
                if($stock_quantity < 0) {
                    $stock_quantity = 0;
                }
                $item->stock_quantity($stock_quantity);
            }
    
            require MT::Entry;
            $item->active($entry->status == MT::Entry::RELEASE() ? 1 : 0);

            unless($item->save) {
                my $message = translate('Saving [_1] failed: [_2]', class_label(), $item->errstr);
                die $message;
            }
        }
    };
    # unlock() 
    $class->driver->sql("UNLOCK TABLES");

    if($@) {
        return $cb->error($@);
    }
    return 1;
}

1;
__END__


